@@ -10,15 +10,16 @@ $OUTPUT_AUTOFLUSH=1;
 
  
 use vars qw($VERSION);
-$VERSION = '0.1';
+$VERSION = '0.2';
 
 my $VERBOSE = 0;
 
 use Readonly;
 
 Readonly my $EMPTY_STRING => q{};
-Readonly my %IS_GRAPHML_WEIGHT_ATTR => map { ($_ => 1) } qw(weight value cost distance );
+Readonly my %IS_GRAPHML_WEIGHT_ATTR => map { ($_ => 1) } qw(weight value cost distance height);
 Readonly my %IS_GRAPHML_LABEL_ATTR => map { ($_ => 1) } qw(label name description nlabel);
+Readonly my $PINF => 1e9999;         # positive infinity
 	
 ## no critic (PostfixControls)
 
@@ -47,8 +48,7 @@ sub VERBOSE {
 sub _initialize {
 	my $self = shift;
 	$self->{graph} = ();
-	$self->{maxcost} = 0;
-	$self->{addOne} = 0;
+	#expect to add other code here
 	return $self;
 }
 
@@ -70,12 +70,17 @@ sub new {
 sub node {
 	my ($self, $nodeA, $label) = @_;
 	
+	croak "node: missing nodeID parameter" if !defined($nodeA);
+	
 	if (defined($label)) {
-		$self->{graph}{$nodeA}{label} = $label;
-		return $self;
+		if (ref(\$nodeA) eq 'SCALAR') {
+			$self->{graph}{$nodeA}{label} = $label;
+			return $self;
+		}
+		croak "node: nodeID is not a SCALAR value";
 	}
 	
-	if (defined($nodeA) and exists($self->{graph}{$nodeA})) {
+	if (exists($self->{graph}{$nodeA})) {
 		return $self->{graph}{$nodeA}{label};
 	}
 	
@@ -85,12 +90,9 @@ sub node {
 sub nodeExists {
 	my ($self, $nodeA) = @_;
 	
-	croak "nodeExists: missing node parameter" if !defined($nodeA);
+	croak "nodeExists: missing nodeID parameter" if !defined($nodeA);
 	
-	if (defined($nodeA) and exists($self->{graph}{$nodeA})) {
-		return 1;
-	}
-	return 0;
+	return (exists($self->{graph}{$nodeA})) ? 1 : 0;
 }
 
 
@@ -108,7 +110,7 @@ sub nodeList {
 sub removeNode {
 	my ($self, $nodeA) = @_;
 	
-	croak "removeNode: missing node parameter" if !defined($nodeA);
+	croak "removeNode: missing nodeID parameter" if !defined($nodeA);
 	
 	if (exists($self->{graph}{$nodeA})) {
 		if (exists($self->{graph}{$nodeA}{edges})) {
@@ -144,8 +146,6 @@ sub edge {
 		$self->{graph}{$nodeA}{edges}{$nodeB} = $cost;
 		$self->{graph}{$nodeB}{edges}{$nodeA} = $cost;
 			
-		$self->{maxcost} += $cost;
-		$self->{addOne} = 0;
 		return($self);
 	}
 	else {
@@ -170,7 +170,7 @@ sub removeEdge {
 	my ($self, $nodeA, $nodeB) = @_;
 	
 	if (!defined($nodeA) or !defined($nodeB)) {
-		croak "removeEdge: missing parameters";
+		croak "removeEdge: missing nodeID parameters";
 	}
 		
 	if (exists($self->{graph}{$nodeA}{edges}{$nodeB})) {
@@ -196,14 +196,10 @@ sub edgeExists {
 	my ($self, $nodeA, $nodeB) = @_;
 	
 	if (!defined($nodeA) or !defined($nodeB) ) {
-		carp "edgeExists: missing nodeID parameters";
-		return;
+		croak "edgeExists: missing nodeID parameters";
 	}
 	
-	if (exists($self->{graph}{$nodeA}{edges}{$nodeB})) {
-		return 1;
-	}
-	return 0;
+	return (exists($self->{graph}{$nodeA}{edges}{$nodeB})) ? 1 : 0;
 }
 
 
@@ -211,15 +207,10 @@ sub adjacent {
 	my ($self, $nodeA, $nodeB) = @_;
 	
 	if (!defined($nodeA) or !defined($nodeB) ) {
-		carp "adjacent: missing nodeID parameters";
-		return;
+		croak "adjacent: missing nodeID parameters";
 	}
 	
-	if ( !exists($self->{graph}{$nodeA}{edges}) or !exists($self->{graph}{$nodeB}{edges}) or !exists($self->{graph}{$nodeA}{edges}{$nodeB}) ) {
-		return 0;
-	}
-	
-	return 1;
+	return ( exists($self->{graph}{$nodeA}{edges}{$nodeB}) ) ? 1 : 0;
 }
 
 
@@ -227,8 +218,7 @@ sub adjacentNodes {
 	my ($self, $nodeA) = @_;
 	
 	if (!defined($nodeA)) {
-		carp "adjacentNodes: missing nodeID parameter";
-		return;
+		croak "adjacentNodes: missing nodeID parameter";
 	}
 	
 	my @neighbors = ();
@@ -246,6 +236,161 @@ sub adjacentNodes {
 #Dijkstra Computation Methods                                               #
 #############################################################################
 
+#sub setSolutionMatrix {
+#	my ($solutionMatrix, $origin, $destination, $cost) = @_;
+#	$solutionMatrix->{row}{$origin}{$destination} = $cost;
+#	$solutionMatrix->{row}{$destination}{$origin} = $cost;
+#}
+
+#Computes Jordan center by creating all pairs shortest path matrix
+
+sub vertexCenter {
+	my ($self, $solutionMatrix) = @_;
+	
+	%$solutionMatrix = ();
+	
+	my @connectedNodeList = ();
+	
+	my $totalNodes = 0;
+	foreach my $nodeID ( keys %{$self->{graph}} ) {
+		$totalNodes++;
+		next if !exists($self->{graph}{$nodeID}{edges});	#exclude disconnected nodes.
+		push(@connectedNodeList, $nodeID);
+	}
+	print "graph contains ", scalar(@connectedNodeList), " connected nodes (nodes >= 1 edges)\n" if $VERBOSE;
+	carp "excluded one or more disconnected nodes (nodes with no edges)" if (scalar(@connectedNodeList) < $totalNodes);
+	
+	foreach my $fromNodeID (@connectedNodeList) {
+		
+		$solutionMatrix->{rowMax}{$fromNodeID} = $PINF;
+		
+		foreach my $toNodeID (@connectedNodeList) {
+			$solutionMatrix->{row}{$fromNodeID}{$toNodeID} = $PINF;
+		}
+		$solutionMatrix->{row}{$fromNodeID}{$fromNodeID} = 0;
+	}
+	
+	
+	my $cycle = 0;
+	my $t0 = Benchmark->new;
+	
+	foreach my $origin (@connectedNodeList) {
+		foreach my $destination (@connectedNodeList) {
+			
+			next if $solutionMatrix->{row}{$origin}{$destination} < $PINF or $origin eq $destination;
+			#print "shortest path $origin -> $destination...";
+			
+			my $pq = Array::Heap::ModifiablePriorityQueue->new();	
+			
+			my %solution = ();
+			my %unvisited = ();
+			foreach my $node (@connectedNodeList) {
+				$solution{$node}{cost} = $PINF;
+			#	$solution{$node}{prevnode} = $EMPTY_STRING;  #not sure if this is needed
+				$pq->add($node, $PINF);
+			}
+				
+			$solution{$origin}{cost} = 0;
+			$pq->add($origin,0); #modify weight of origin node
+			
+			
+			#my $foundSolution = 0;
+			while ($pq->size()) {
+				$cycle++;
+				
+				my $visitNode = $pq->get();
+				
+				#setSolutionMatrix($solutionMatrix, $origin, $visitNode, $solution{$visitNode}{cost});
+				$solutionMatrix->{row}{$origin}{$visitNode} = $solution{$visitNode}{cost};
+				$solutionMatrix->{row}{$visitNode}{$origin} = $solution{$visitNode}{cost};
+				
+				last if ($visitNode eq $destination);
+				
+				foreach my $adjacentNode (keys %{$self->{graph}{$visitNode}{edges}}) {
+					next if !defined($pq->weight($adjacentNode));
+					
+					my $thisCost = $solution{$visitNode}{cost} + $self->{graph}{$visitNode}{edges}{$adjacentNode};
+					if ($thisCost < $solution{$adjacentNode}{cost}) {
+						$solution{$adjacentNode}{cost} = $thisCost;
+					#	$solution{$adjacentNode}{prevnode} = $visitNode;
+						$pq->add($adjacentNode, $thisCost);
+					}
+				}
+			}
+				
+			undef($pq);
+		}
+	}
+	if ($VERBOSE) {
+		my $t1 = Benchmark->new;
+		#if ($cycle >= 1000) {
+		#	print "\n";
+		#}
+		my $td = timediff($t1, $t0);
+	  print "computing shortest path matrix took: ",timestr($td),"\n";
+	}
+	my $graphMinMax = $PINF;
+	my $centerNode = '';
+	foreach my $origin (@connectedNodeList) {
+		my $rowMax = 0;
+		foreach my $destination (@connectedNodeList) {
+			next if $origin eq $destination;
+			if ($solutionMatrix->{row}{$origin}{$destination} > $rowMax) {
+				$rowMax = $solutionMatrix->{row}{$origin}{$destination};
+			}
+		}
+		$solutionMatrix->{rowMax}{$origin} = $rowMax;
+		if ($rowMax < $graphMinMax) {
+			$graphMinMax = $rowMax;
+		}
+	}
+	$solutionMatrix->{centerNodeSet} = [];
+	if ($graphMinMax < $PINF) {
+		foreach my $origin (@connectedNodeList) {
+			if ($solutionMatrix->{rowMax}{$origin} == $graphMinMax) {
+				push(@{$solutionMatrix->{centerNodeSet}}, $origin);
+			}
+		}
+	}
+	else {
+		carp "Graph contains disconnected sub-graph. Center node set undefined.";
+		$graphMinMax = 0;
+	}
+	#print "centernodeset ", join(',', @{$solutionMatrix->{centerNodeSet}}), "\n";
+	return($graphMinMax);
+}
+
+sub outputAPSPmatrixtoCSV {
+	my ($either, $solutionMatrix, $filename, $labelSort) = @_;
+	
+	use Text::CSV_XS;
+	$labelSort = '' if !defined($labelSort);
+	
+	open(my $outfile, '>:encoding(UTF-8)', $filename) or croak "could not open '$filename'";
+	
+	print "outputAPSPmatrixtoCSV: opened '$filename' for output\n" if $VERBOSE;
+	
+	my $csv = Text::CSV_XS->new ({ binary => 1, auto_diag => 1 });
+	
+	my @nodeList = (lc($labelSort) eq 'numeric') ? (sort {$a <=> $b} keys %{$solutionMatrix->{row}}) : (sort keys %{$solutionMatrix->{row}});
+	
+	$csv->say($outfile, ['From/To', @nodeList ]);
+	my $rowcount = 1;
+	
+	foreach my $nodeID (@nodeList) {
+		my @row = ();
+		foreach my $destinationID (@nodeList) {
+			push(@row, $solutionMatrix->{row}{$nodeID}{$destinationID});
+		}
+		$csv->say($outfile, [$nodeID, @row]);
+		$rowcount++;
+	}
+	close($outfile);
+	print "outputAPSPmatrixtoCSV: wrote $rowcount rows to '$filename'\n" if $VERBOSE;
+	return $either;
+	
+}
+
 
 sub farthestNode {  ## no critic (ProhibitExcessComplexity)
 	my ($self, $origin, $solutionHref) = @_;
@@ -265,18 +410,13 @@ sub farthestNode {  ## no critic (ProhibitExcessComplexity)
 	}
 	my $pq = Array::Heap::ModifiablePriorityQueue->new();
 	
-	if (!$self->{addOne}) {
-		$self->{maxcost} += 1;
-		$self->{addOne} = 1;
-	}
-	
-	my %solution = ();
+	my %solution = ();		#initialize the solution hash
 	my %unvisited = ();
 	foreach my $node (keys %{$self->{graph}}) {
 		if (exists($self->{graph}{$node}{edges})) {  #nodes without edges cannot be part of the solution
-			$solution{$node}{cost} = $self->{maxcost};
+			$solution{$node}{cost} = $PINF;
 			$solution{$node}{prevnode} = $EMPTY_STRING;
-			$pq->add($node, $self->{maxcost});
+			$pq->add($node, $PINF);
 		}
 	}
 		
@@ -314,7 +454,7 @@ sub farthestNode {  ## no critic (ProhibitExcessComplexity)
   
 	my $farthestcost = 0;
 	foreach my $node (sort keys %solution) {
-		if ($solution{$node}{cost} < $self->{maxcost} and $solution{$node}{cost} > $farthestcost) {
+		if ($solution{$node}{cost} < $PINF and $solution{$node}{cost} > $farthestcost) {
 			$farthestcost = $solution{$node}{cost};
 			#$farthestnode = $node;
 		}
@@ -416,18 +556,13 @@ sub shortestPath { ## no critic (ProhibitExcessComplexity)
 	}
 	my $pq = Array::Heap::ModifiablePriorityQueue->new();
 	
-	if (!$self->{addOne}) {
-		$self->{maxcost} += 1;
-		$self->{addOne} = 1;
-	}
-	
-	my %solution = ();
+	my %solution = ();		#initialize the solution hash
 	my %unvisited = ();
 	foreach my $node (keys %{$self->{graph}}) {
 		if (exists($self->{graph}{$node}{edges})) {  #nodes without edges cannot be part of the solution
-			$solution{$node}{cost} = $self->{maxcost};
+			$solution{$node}{cost} = $PINF;
 			$solution{$node}{prevnode} = $EMPTY_STRING;
-			$pq->add($node, $self->{maxcost});
+			$pq->add($node, $PINF);
 		}
 	}
 		
@@ -445,7 +580,7 @@ sub shortestPath { ## no critic (ProhibitExcessComplexity)
 		my $visitNode = $pq->get();
 		
 		if ($visitNode eq $destination) {
-			$foundSolution = 1 if $solution{$destination}{cost} < $self->{maxcost};
+			$foundSolution = 1 if $solution{$visitNode}{cost} < $PINF;
 			last;
 		}
 		
@@ -539,15 +674,7 @@ sub inputGraphfromCSV {
 	carp "inputGraphfromCSV: no nodes read from '$filename'" if !$nodecount;
 	carp "inputGraphfromCSV: no edges read from '$filename'" if !$edgecount;
 	
-	if ($edgecount) {
-		$self->{maxcost} += 1;
-		$self->{addOne} = 1;
-	}
-	
-	if ($VERBOSE) {
-		print "inputGraphfromCSV: found $nodecount nodes and $edgecount edges\n";
-		print "inputGraphfromCSV: maxcost = $self->{maxcost}\n";
-	}
+	print "inputGraphfromCSV: found $nodecount nodes and $edgecount edges\n" if $VERBOSE;
 	return $self;
 }
 
@@ -592,15 +719,8 @@ sub inputGraphfromJSON {
 	carp "inputGraphfromJSON: no nodes read from '$filename'" if !$nodecount;
 	carp "inputGraphfromJSON: no edges read from '$filename'" if !$edgecount;
 	
-	if ($edgecount) {
-		$self->{maxcost} += 1;
-		$self->{addOne} = 1;
-	}
+	print "inputGraphfromJSON: found $nodecount nodes and $edgecount edges\n" if $VERBOSE;
 	
-	if ($VERBOSE) {
-		print "inputGraphfromJSON: found $nodecount nodes and $edgecount edges\n";
-		print "inputGraphfromJSON: maxcost = $self->{maxcost}\n";
-	}
 	return $self;
 }
 
@@ -685,15 +805,8 @@ sub inputGraphfromGML { ## no critic (ProhibitExcessComplexity)
 	carp "inputGraphfromGML: no nodes read from '$filename'" if !$nodecount;
 	carp "inputGraphfromGML: no edges read from '$filename'" if !$edgecount;
 	
-	if ($edgecount) {
-		$self->{maxcost} += 1;
-		$self->{addOne} = 1;
-	}
+	print "inputGraphfromGML: found $nodecount nodes and $edgecount edges\n" if $VERBOSE;
 	
-	if ($VERBOSE) {
-		print "inputGraphfromGML: found $nodecount nodes and $edgecount edges\n";
-		print "inputGraphfromGML: maxcost = $self->{maxcost}\n";
-	}
 	return $self;
 }
 
@@ -825,15 +938,8 @@ sub inputGraphfromGraphML { ## no critic (ProhibitExcessComplexity)
 	carp "inputGraphfromGraphML: no nodes read from '$filename'" if !$nodecount;
 	carp "inputGraphfromGraphML: no edges read from '$filename'" if !$edgecount;
 	
-	if ($edgecount) {
-		$self->{maxcost} += 1;
-		$self->{addOne} = 1;
-	}
+	print "inputGraphfromGraphML: found $nodecount nodes and $edgecount edges\n" if $VERBOSE;
 	
-	if ($VERBOSE) {
-		print "inputGraphfromGraphML: found $nodecount nodes and $edgecount edges\n";
-		print "inputGraphfromGraphML: maxcost = $self->{maxcost}\n";
-	}
 	return $self;
 }
 
@@ -1066,7 +1172,7 @@ __END__
 
 =head1 NAME
  
-Graph::Dijkstra - Dijkstra's shortest path algorithm with methods to input/output graph datasets using supported file formats
+Graph::Dijkstra - Dijkstra's shortest path algorithm with methods to input/output graph datasets from/to supported file formats
  
 =for comment '
 
@@ -1116,16 +1222,40 @@ Graph::Dijkstra - Dijkstra's shortest path algorithm with methods to input/outpu
   	print Dumper(\%Solution);
   }
   
-  #compute shortest path between two nodes (from origin to destination)
+  #shortest path between two nodes (from origin to destination)
   if (my $pathCost = $graph->shortestPath( 0, 2, \%Solution ) ) {
   	print Dumper(\%Solution);
   }
   
+  #Jordan or vertex center with all points shortest path matrix
+  my %solutionMatrix = ();
+  if (my $graphMinMax = $graph->vertexCenter(\%solutionMatrix) ) {
+  	print "Center Node Set 'eccentricity', minimal greatest distance to all other nodes $graphMinMax\n";
+  	print "Center Node Set = ", join(',', @{$solutionMatrix{centerNodeSet}} ), "\n";
+  	
+  	my @nodeList = (sort keys %{$solutionMatrix{row}});
+  	print 'From/To,', join(',',@nodeList), "\n";
+  	foreach my $fromNode (@nodeList) {
+  		print "$fromNode";
+  		foreach my $toNode (@nodeList) {
+  			print ",$solutionMatrix{row}{$fromNode}{$toNode}";
+  		}
+  		print "\n";
+  	}
+  	$graph->outputAPSPmatrixtoCSV(\%solutionMatrix, 'APSP.csv');
+  }
+  
 =head1 DESCRIPTION
  
 Efficient implementation of Dijkstras shortest path algorithm in Perl using a Minimum Priority Queue (L<Array::Heap::ModifiablePriorityQueue>**).
 
-Includes methods to input and output graph datasets from/to the following file formats.
+Computation methods.
+
+	farthestNode() Shortest path to farthest node (from an origin node) C<$graph->farthestNode()>
+	shortestPath() Shortest path between two nodes
+	vertexCenter() Jordan center node set (vertex center) with all points shortest path (APSP) matrix
+
+Methods that input and output graph datasets from/to the following file formats.
 
 	GML (Graph Modelling Language, not to be confused with Geospatial Markup Language), 
 	JSON Graph Specification (latest draft specification, edge weights/costs input using metadata "value" attribute), 
@@ -1396,6 +1526,45 @@ Produces the following output
 		source='H' target='D' cost='5'
 		source='D' target='A' cost='3'
 
+=item my $graphMinMax = $graph->vertexCenter($solutionMatrixHref);
+
+Returns the graph "eccentricity", the minimal greatest distance to all other nodes from the "center node" set or Jordan center.
+Carps if graph contains disconnected nodes (nodes with no edges) which are excluded.  If graph contains a disconnected sub-graph (a set of connected
+nodes isoluated / disconnected from all other nodes), the return value is 0 -- as the center nodes are undefined.
+
+The $solutionMatrix hash (reference) is updated to include the center node set (the list of nodes with the minimal greatest distance
+to all other nodes) and the all points shortest path matrix.  In the all points shortest path matrix, an infinite value (1.#INF) indicates that there
+is no path from the origin to the destination node.  In this case, the center node set is empty and the return value is 0.
+
+Includes a class "helper" method that outputs the All Points Shortest Path matrix to a CSV file.
+
+NOTE: The size of the All Points Shortest Path matrix is nodes^2 (expontial).  A graph with a thousand nodes results in a million entry matrix that
+will take a long time to compute.  Have not evaluated the practical limit on the number of nodes.
+
+Sample code
+
+	my %solutionMatrix = ();
+	
+	my $graphMinMax = $graph->vertexCenter(\%solutionMatrix);
+	print "Center Node Set = ", join(',', @{$solutionMatrix{centerNodeSet}} ), "\n";
+	print "Center Node Set 'eccentricity', minimal greatest distance to all other nodes $graphMinMax\n";
+	
+	my @nodeList = (sort keys %{$solutionMatrix{row}});
+	#  or (sort {$a <=> $b} keys %{$solutionMatrix{row}}) if the $nodeID values are numeric
+	
+	print 'From/To,', join(',',@nodeList), "\n";
+	foreach my $fromNode (@nodeList) {
+		print "$fromNode";
+		foreach my $toNode (@nodeList) {
+			print ",$solutionMatrix{row}{$fromNode}{$toNode}";
+		}
+		print "\n";
+	}
+	
+	# Output All Points Shortest Path matrix to a .CSV file.  
+	# If the nodeID values are numeric, include a third parameter, 'numeric' to sort the nodeID values numerically.
+	
+	$graph->outputAPSPmatrixtoCSV(\%solutionMatrix, 'APSP.csv');
 
 =back
 
@@ -1535,7 +1704,8 @@ Update $graph->inputGraphfrom*, $graph->outputGraphto*, $graph->node*, and $grap
 
 Support input, Dijkstra path computations, and output of directed graphs (graphs with directed / uni-directional edges) in a comprehensive manner.
 
-Add computation methods.
+Evaluate vertexCenter method with larger graphs.  Current implementation uses Dijkstra shortest path algorithm. 
+Possibly replace with Floyd Warshall algorithm or add second implementation.
 
 Support additional graph file formats.
 
